@@ -3,10 +3,9 @@
 """Data structures for Zenodo."""
 
 import datetime
-from dataclasses import dataclass, field
-from typing import Optional, Sequence
+from typing import Literal, Optional, Sequence
 
-from dataclasses_json import DataClassJsonMixin
+from pydantic import BaseModel, Field
 
 __all__ = [
     "Creator",
@@ -14,26 +13,38 @@ __all__ = [
 ]
 
 
-@dataclass
-class Creator(DataClassJsonMixin):
-    """A creator."""
+# https://developers.zenodo.org/#rest-api
 
-    #: Name of the creator in the format Family name, given names
-    name: str
-    #: Affiliation of the creator
-    affiliation: Optional[str] = None
-    #: ORCID identifier of the creator
-    orcid: Optional[str] = None
 
-    #: GND identifier of creator
-    # gnd: Optional[str] = None
+class Creator(BaseModel):
+    """A creator, see https://developers.zenodo.org/#representation."""
+
+    name: str = Field(
+        ..., description="Name of the creator in the format Family name, given names", example="Hoyt, Charles Tapley"
+    )
+    affiliation: Optional[str] = Field(description="affiliation of the creator", example="Harvard Medical School")
+    orcid: Optional[str] = Field(description="ORCID identifier of the creator", example="0000-0003-4423-4370")
+    gnd: Optional[str] = Field(
+        description="German National Library identifier of the creator. "
+        "See also https://www.wikidata.org/wiki/Property:P227."
+    )
+
+    @property
+    def orcid_url(self) -> Optional[str]:
+        """Get the ORCID identifier as a URL."""
+        return f"https://orcid.org/{self.orcid}" if self.orcid else None
+
+    @property
+    def gnd_url(self) -> Optional[str]:
+        """Get the GND identifier as a URL."""
+        return f"https://d-nb.info/gnd/{self.gnd}"
 
     def __post_init__(self):  # noqa:D105
         if "," not in self.name:
             raise ValueError("name should be in format Family name, given names")
 
 
-ALLOWED_TYPES = {
+UploadType = Literal[
     "publication",
     "poster",
     "presentation",
@@ -44,9 +55,9 @@ ALLOWED_TYPES = {
     "lesson",
     "physicalobject",
     "other",
-}
+]
 
-ALLOWED_PUBLICATION_TYPES = {
+PublicationType = Literal[
     "annotationcollection",
     "book",
     "section",
@@ -65,52 +76,49 @@ ALLOWED_PUBLICATION_TYPES = {
     "thesis",
     "workingpaper",
     "other",
-}
+]
 
-VALID_IMAGE_TYPES = {
+ImageType = Literal[
     "figure",
     "plot",
     "drawing",
     "diagram",
     "photo",
     "other",
-}
-ALLOWED_ACCESS_RIGHT = {
+]
+AccessRight = Literal[
     "open",
     "embargoed",
     "restricted",
     "closed",
-}
+]
 
 
-@dataclass
-class Metadata(DataClassJsonMixin):
+def _today_str() -> str:
+    return datetime.datetime.today().strftime("%Y-%m-%d")
+
+
+class Metadata(BaseModel):
     """Metadata for the Zenodo deposition API."""
 
     title: str
-    upload_type: str
+    upload_type: UploadType
     description: str
     creators: Sequence[Creator]
-    access_right: str = "open"
+    access_right: AccessRight = "open"
     language: Optional[str] = "eng"
-    version: Optional[str] = field(default_factory=lambda: datetime.datetime.today().strftime("%Y-%m-%d"))
+    version: Optional[str] = Field(default_factory=_today_str)
     license: Optional[str] = "CC0-1.0"
-    publication_type: Optional[str] = None
-    image_type: Optional[str] = None
+    publication_type: Optional[PublicationType] = None
+    image_type: Optional[ImageType] = None
 
     def __post_init__(self):  # noqa:D105
-        if self.upload_type not in ALLOWED_TYPES:
-            raise ValueError(f"invalid upload_type: {self.upload_type}")
         if self.upload_type == "publication":
             if self.publication_type is None:
                 raise ValueError("missing publication_type")
-            if self.publication_type not in ALLOWED_PUBLICATION_TYPES:
-                raise ValueError(f"invalid publication_type: {self.publication_type}")
         elif self.upload_type == "image":
             if self.image_type is None:
                 raise ValueError("missing image_type")
-            if self.image_type not in VALID_IMAGE_TYPES:
-                raise ValueError(f"invalid image_type: {self.image_type}")
         if self.access_right in {"open", "embargoed"}:
             if self.license is None:
                 raise ValueError(f"need a license for access_right={self.access_right}")
